/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>



#define VER         "0.1.1"
#define COFFEEDATA  "robert249fsd)af8.?sf2eaya;sd$%85034gsn%@#!afsgsjdg;iawe;otigkbarr"
#define KEYPATH     "Software\\CoffeeCup Software\\Internet\\Profiles\\"
#define PROXYPATH   "Software\\CoffeeCup Software\\Internet\\Proxy"
#define OUTFILE     "coffecup-regdump.txt"
#define BUFFSZ      512



void coffeeftp_pwdencdec(u_char *data);
void std_err(void);



int main(void) {
    FILE    *fd;
    HKEY    key,
            keyvalue;
    u_char  keybuff[sizeof(KEYPATH) + BUFFSZ],
            buff[BUFFSZ],
            value[BUFFSZ],
            *ptr;
    int     bufflen,
            valuelen,
            k,
            v;


    setbuf(stdout, NULL);

    fputs("\n"
        "CoffeeCup FTP profiles collector and decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    fputs("\nCreating file "OUTFILE" (it will contain all the dumped data)\n", stdout);
    fd = fopen(OUTFILE, "wb");
    if(!fd) std_err();

    if(RegOpenKeyEx(HKEY_CURRENT_USER, KEYPATH, 0, KEY_READ, &key)) {
        fputs("\n"
            "Error: registry key has not been found\n"
            "\n"
            "    Press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    memcpy(keybuff, KEYPATH, sizeof(KEYPATH));
    ptr = keybuff + sizeof(KEYPATH) - 1;
    k = 0;
    for(;;) {
        bufflen = BUFFSZ;
        if(RegEnumKeyEx(
            key,
            k++,
            ptr,
            (void *)&bufflen,
            NULL,
            NULL,
            NULL,
            NULL)) break;

        fprintf(fd, "\n"
            "---------------------\n"
            "Profile: %s\n"
            "---------------------\n", ptr);

        RegOpenKeyEx(HKEY_CURRENT_USER, keybuff, 0, KEY_READ, &keyvalue);
        v = 0;
        for(;;) {
            bufflen = BUFFSZ;
            if(RegEnumValue(
                keyvalue,
                v++,
                buff,
                (void *)&bufflen,
                NULL,
                NULL,
                NULL,
                NULL)) break;
            fprintf(fd, "%-20s ", buff);

            valuelen = BUFFSZ;
            if(RegQueryValueEx(
                keyvalue,
                buff,
                NULL,
                NULL,
                value,
                (void *)&valuelen)) break;

            if(!stricmp(buff, "password")) coffeeftp_pwdencdec(value);
            fprintf(fd, "%s\n", value);
        }
        RegCloseKey(keyvalue);
    }
    RegCloseKey(key);



        /* PROXY */

    if(!RegOpenKeyEx(HKEY_CURRENT_USER, PROXYPATH, 0, KEY_READ, &keyvalue)) {
        fputs("\n"
            "---------------------\n"
            "PROXY informations\n"
            "---------------------\n", fd);
        v = 0;
        for(;;) {
            bufflen = BUFFSZ;
            if(RegEnumValue(
                keyvalue,
                v++,
                buff,
                (void *)&bufflen,
                NULL,
                NULL,
                NULL,
                NULL)) break;
            fprintf(fd, "%-20s ", buff);

            valuelen = BUFFSZ;
            if(RegQueryValueEx(
                keyvalue,
                buff,
                NULL,
                NULL,
                value,
                (void *)&valuelen)) break;

            if(!stricmp(buff, "proxypassword")) coffeeftp_pwdencdec(value);
            fprintf(fd, "%s\n", value);
        }
        RegCloseKey(keyvalue);
    }

    fclose(fd);
    fputs("\n"
        "Finished\n"
        "\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void coffeeftp_pwdencdec(u_char *data) {
    u_char  ah,
            cnt;
    int     len;

    len = strlen(data);
    ah = cnt = 0x40;
    while(len--) {
        if(!(ah & 8)) ah ^= 1;
        ah ^= 0xff;
        ah = (ah & 1) ? ((ah >> 1) | 0x80) : (ah >> 1); // ROR ah, 1
        *data++ ^= (COFFEEDATA[cnt] ^ ah);
        if(!--cnt) cnt = 0x40;
    }
}



void std_err(void) {
    perror("\nError");
    fputs("\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}


